@@ -4,16 +4,13 @@ by a monitor for LSR.
 
 @author: Peter Parente
 @organization: IBM Corporation
-@copyright: Copyright (c) 2005, 2006 IBM Corporation
-@license: Common Public License 1.0
+@copyright: Copyright (c) 2005, 2007 IBM Corporation
+@license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made 
-available under the terms of the Common Public License v1.0 which accompanies
+available under the terms of the BSD license which accompanies
 this distribution, and is available at
-U{http://www.opensource.org/licenses/cpl1.0.php}
+U{http://www.opensource.org/licenses/bsd-license.php}
 '''
 from Base import AEMonitor
-from EventMonitor import EventMonitor
-from InputMonitor import InputMonitor
-from OutputMonitor import OutputMonitor
 from MonitorCollection import MonitorCollection
\ No newline at end of file
