@@ -70,10 +70,9 @@ class DefaultAccInfoAdapter(PORAdapter):
       attributes
     '''
     try:
-      attribute_string = self.accessible.getAttributes()
+      attribute_list = self.accessible.getAttributes()
     except (AttributeError, Constants.NotImplemented):
       raise NotImplementedError
-    attribute_list = attribute_string.split('; ')
     return attribute_list
 
   @pyLinAcc.errorToLookupError
@@ -259,14 +258,39 @@ class DefaultAccInfoAdapter(PORAdapter):
     Gets the visual position of the focal point wihin an accessible object,
     the center in a simple box-like control.
 
+    @return: x,y coordinate
+    @rtype: 2-tuple integer
+    @raise NotImplementedError: When this accessible does not support the
+      Component interface
+    '''
+    try:
+      text = Interfaces.IText(self.accessible)
+    except NotImplementedError:
+      text = None
+      
+    if text:
+      # use the character extents to track to the current character
+      off = self.char_offset
+      x, y, w, h = text.getCharacterExtents(off, Constants.DESKTOP_COORDS)
+      return x, y
+    else:
+      # just center on the widget
+      c = Interfaces.IComponent(self.accessible)
+      bb = c.getExtents(Constants.DESKTOP_COORDS)
+      return bb.x+bb.width/2, bb.y+bb.height/2
+  
+  @pyLinAcc.errorToLookupError
+  def getAccPosition(self):
+    '''
+    Gets the position of the accessible object, usually upper-left corner.
+    
     @return: x,y coordinate
     @rtype: 2-tuple integer
     @raise NotImplementedError: When this accessible does not support the
       Component interface
     '''
     c = Interfaces.IComponent(self.accessible)
-    bb = c.getExtents(Constants.DESKTOP_COORDS)
-    return bb.x+bb.width/2, bb.y+bb.height/2
+    return c.getPosition(Constants.DESKTOP_COORDS)
 
   @pyLinAcc.errorToLookupError
   def getAccTextAttr(self, name):
@@ -477,7 +501,7 @@ class DefaultAccInfoAdapter(PORAdapter):
     @raise LookupError: When the accessible or any parent accessible up to the
       application is dead
     '''
-    app = self.accessible.getApplication()
+    app = Interfaces.IApplication(self.accessible.getApplication())
     return app.getLocale(Constants.LOCALE_TYPE_MESSAGES)
 
   @pyLinAcc.errorToLookupError
@@ -587,6 +611,7 @@ class DefaultAccInfoAdapter(PORAdapter):
     @rtype: list
     @raise LookupError: When the accessible object is dead
     '''
+    role = self.accessible.getRoleName()
     states = set([pyLinAcc.stateToString(val) for val in 
                   self.accessible.getState().getStates()])
     # collapsed isn't always set properly, so add it if expandable but not
@@ -594,11 +619,12 @@ class DefaultAccInfoAdapter(PORAdapter):
     if 'expandable' in states and 'expanded' not in states:
       states.add('collapsed')
     # give some indicator of not being enabled
-    if 'enabled' not in states and self.getAccRoleName() != 'icon':
+    if ('enabled' not in states and role != 'icon' and 
+        ('focusable' in states or 'editable' in states or 
+         'selectable' in states)):
       states.add('disabled')
     # give an indication of not checked
-    if (self.accessible.getRoleName().find('check') > -1 and 
-        'checked' not in states):
+    if (role.find('check') > -1 and 'checked' not in states):
       states.add('unchecked')
     return states
   
