@@ -20,7 +20,7 @@ __uie__ = dict(kind='perk', all_tiers=True)
 
 class DefaultDialogPerk(Perk.Perk):
   '''
-  Manages the built in LSR dialogs (e.g. settings).
+  Manages the basic LSR dialogs for configuring settings and loading Perks.
   
   Shows and hides default LSR chooser dialogs for configuring LSR settings,
   loading and unloading extensions at run time, and so on. Tasks for starting 
@@ -37,6 +37,9 @@ class DefaultDialogPerk(Perk.Perk):
     self.registerTask(PerkChooserTask('show perk chooser'))
     self.registerTask(SettingsChooserTask('show settings chooser'))
     
+  def getName(self):
+    return _('Basic dialogs')
+    
 class SettingsChooserTask(Task.ChooserTask):
   '''  
   Shows the settings chooser which allows a user to configure LSR devices,
@@ -335,12 +338,11 @@ class PerkChooserTask(Task.ChooserTask):
       kwargs['app'] = self.getAppName()
       text = _('removed %(name)s from %(app)s') % kwargs
       self.doTask('read message', text=text)
+
     if kind in (chooser.OK, chooser.APPLY):
       # load and unload Perks
       self.doPerkLoad(**kwargs)
-    elif kind == chooser.CANCEL:
-      # just quit
-      return
+
     
   def doPerkLoad(self, loaded, unloaded, **kwargs):
     '''
