@@ -62,7 +62,7 @@ on message received: [text of message]
 @author: Brett Clippingdale
 @author: Peter Parente
 @organization: IBM Corporation
-@copyright: Copyright (c) 2005, 2006 IBM Corporation
+@copyright: Copyright (c) 2005, 2007 IBM Corporation
 @license: The BSD License
 
 All rights reserved. This program and the accompanying materials are made 
@@ -70,7 +70,7 @@ available under the terms of the BSD license which accompanies
 this distribution, and is available at
 U{http://www.opensource.org/licenses/bsd-license.php}
 '''
-import Perk, Task, AEState
+import Perk, Task, AEState, AEConstants
 from POR import POR
 from i18n import _
 
@@ -112,23 +112,23 @@ class GaimPerkState(Perk.PerkState):
   '''
   Settings for Gaim.
   
-  @cvar ReadBackground: Read incoming messages when gaim is in the background?
-  @type ReadBackground: boolean
+  ReadBackground (bool): Read incoming messages when gaim is in the background?
   '''
-  ReadBackground = False
+  def init(self):
+    self.newBool('ReadBackground', False, _('Read background messages?'), 
+                 _('When checked, incoming messages will be read even when '
+                   'Gaim is in the background.'))
   
-  def getSettings(self):
-    g = self._newGroup()
-    g.newBool('ReadBackground', _('Read background messages?'), 
-              _('When checked, incoming messages will be read even when Gaim '
-                'is in the background.'))
+  def getGroups(self):
+    g = self.newGroup()
+    g.append('ReadBackground')
     return g
 
 class GaimPerk(Perk.Perk):
   '''
   Provides hotkeys for switching between the message history and message 
   compose areas. Announces incoming messages and allows for review by entire
-  message. Supplements the DefaultPerk.
+  message.
   
   Has initial support for announcing when buddies sign in and out and when the
   status of a conversation changes. Written to work with both gaim 1.5 and 2.0.
@@ -163,13 +163,13 @@ class GaimPerk(Perk.Perk):
     self.setPerkIdealOutput('audio')    
     
     # register event handlers
-    # events here are app-level, DefaultPerk will handle gnome-level events,
+    # events here are app-level, other Perks will handle gnome-level events,
     # so only register events of interest here. 
-    self.registerTask(ActivateGaimWindow())
-    self.registerTask(HandleFocusChange())
-    self.registerTask(HandleSelectorChange())
-    self.registerTask(HandleCaretChange())
-    self.registerTask(ReadIncoming(all=True))
+    self.registerTask(ActivateGaimWindow('gaim read window'))
+    self.registerTask(HandleFocusChange('gaim read focus'))
+    self.registerTask(HandleSelectorChange('gaim read selector'))
+    self.registerTask(HandleCaretChange('game read caret'))
+    self.registerTask(ReadIncoming('gaim read incoming', all=True))
     # not working for background events ???
     #self.registerEventTask(HandlePropertyChange(focus=True, tier=True,
     #                       background=True))
@@ -178,7 +178,6 @@ class GaimPerk(Perk.Perk):
     # variable) and code in handleCaretChange
     #self.registerEventTask(BuddySignOnOff(focus=True, tier=True,
                            #background=True))
-    # see DefaultPerk.py for more event handling examples
     
     # register named tasks.  Specify the task class name (see below) and a 
     # description.  These descriptions are dictionary keys and therefore
@@ -187,7 +186,7 @@ class GaimPerk(Perk.Perk):
     self.registerTask(ConversationStatus('gaim conversation status'))
 
     # get the Keyboard device and register modifiers.
-    kbd = self.getInputDevice('Keyboard')
+    kbd = self.getInputDevice(None, 'keyboard')
     self.addInputModifiers(kbd, kbd.AEK_CAPS_LOCK)
     
     # register the keys and their associated commands, binding them using the
@@ -285,6 +284,13 @@ class GaimPerk(Perk.Perk):
       print "chat client is Gaim beta 2 or higher"
     else:
       print "chat client is Gaim 1.5 or lower"
+      
+  def getName(self):
+    '''
+    @return: Human readable name of this L{Perk}.
+    @rtype: string
+    '''
+    return _('Gaim')
 
 #
 # Tasks executed by keys
@@ -303,10 +309,12 @@ class ToggleChatPanel(Task.InputTask):
     # Note: ".perk" lets LSR find cross-class variables "global" to the Perk
     tab_list_por = self.perk.getActivePageTabListPOR()
     if tab_list_por is not None:
-      if self.hasAccState('focused', self.perk.chat_compose_por):
+      if not self.perk.is_history_focused:
         #change focus to chat history panel
         self.setAccFocus(self.perk.chat_history_por)  
       else:
+        print 'focus to compose'
+        print self.perk.chat_compose_por
         #change focus to chat compose panel
         self.setAccFocus(self.perk.chat_compose_por) 
     return True # allow triggering keystrokes to propagate (default)
@@ -380,14 +388,13 @@ class ActivateGaimWindow(Task.ViewTask):
     print 'GAIM:', tab_list_por
     # test to see if not a chat window
     if tab_list_por is None:
-      return True # allow DefaultPerk to handle the view change
+      return True # allow other Perks to handle the view change
     # must be a chat window
     else:
       if not self.perk.initialized:
         self.perk.setGaimVersion()
         self.perk.initialized = True
-      # don't allow the view change event to propagate (ie. to DefaultPerk, 
-      # where the frame title would be announced)
+      # don't allow the view change event to propagate
       return False
 
 class HandleFocusChange(Task.FocusTask):
@@ -426,12 +433,13 @@ class HandleFocusChange(Task.FocusTask):
     if self.perk.is_gaim2: 
       self.perk.chat_history_por = \
         self.getAccFromPath(selected_tab_por, *self.GAIM2_HISTORY_PATH)
-      try:
+      self.perk.chat_compose_por = \
+        self.getAccFromPath(selected_tab_por, *self.GAIM2_COMPOSE_PATH)
+      if (self.perk.chat_compose_por is None or
+          not self.hasAccRole('text', self.perk.chat_compose_por)):
         self.perk.chat_compose_por = \
-          self.getAccFromPath(selected_tab_por, *self.GAIM2_COMPOSE_PATH)
-      except:
-        self.perk.chat_compose_por = \
-          self.getAccFromPath(selected_tab_por, *self.GAIM2_ALT_COMPOSE_PATH)      
+          self.getAccFromPath(selected_tab_por, *self.GAIM2_ALT_COMPOSE_PATH)
+        print 'alt', self.perk.chat_compose_por
     # gaim 1.5 local client
     else:
       self.perk.chat_history_por = \
@@ -498,7 +506,8 @@ class ReadIncoming(Task.CaretTask):
     @param layer: Layer of the event, focus, tier, or background
     @type layer: integer
     '''
-    if not self.perk_state.ReadBackground and layer == Task.LAYER_BACKGROUND:
+    if (not self.perk_state.ReadBackground and 
+        layer == AEConstants.LAYER_BACKGROUND):
       # don't read background messages if the user has it disabled
       return
     if text.startswith('\n'):
@@ -516,8 +525,8 @@ class HandleCaretChange(Task.CaretTask):
     '''
     Announces an entire message from a remote user by looking for hard line 
     breaks (\\n) when the caret moves to a position just after or just before
-    the break character. If the caret is anywhere else, lets DefaultPerk do its
-    default caret processing.
+    the break character. If the caret is anywhere else, lets other L{Perk}s do
+    their default caret processing.
     
     @param por: Point of regard where the caret event occurred
     @type por: L{POR}
@@ -546,7 +555,7 @@ class HandleCaretChange(Task.CaretTask):
         # store the current POR
         self.perk.last_history_caret = por
         # if we're not at the start or end of a true message line, let 
-        # DefaultPerk handles the caret event unless at start/end of msg
+        # other Perks handle the caret event unless at start/end of msg
         if por != end and por != start:
           return True
         
@@ -554,12 +563,12 @@ class HandleCaretChange(Task.CaretTask):
         msg = self.getAccTextBetween(start, end)
         
         self.mayStop()
-        self.sayHardline(text=msg)
+        self.sayItem(text=msg)
         #self.inhibitMayStop()
         return False
       
       # respond to caret movement on same line
-      # just let DefaultPerk handle this case
+      # just let other Perks handle this case
       else: 
         self.perk.last_history_caret = por
         return True
@@ -602,7 +611,7 @@ class HandlePropertyChange(Task.PropertyTask):
     '''
     is_chat_window = self.perk.getActivePageTabListPOR()
     # test to see if a chat window and if a conversation page tab
-    if is_chat_window is not None and self.getAccRoleName(por) == "page tab":
+    if is_chat_window is not None and self.hasAccRoleName('page tab', por):
       user = self.getAccName(por)
       status = self.perk.getTypingStatus(por)
       #print "%s: %s: %s" %(name, STATUS_MSG[status], por)
@@ -636,8 +645,7 @@ class BuddySignOnOff(Task.TableTask):
     @type first_child_por: L{POR}
     @param last_child_por: The L{POR} of last inserted row
     @type last_child_por: L{POR}
-    @param layer: Layer on which the event occurred, one of L{Task.LAYER_FOCUS},
-      L{Task.LAYER_TIER}, or L{Task.LAYER_BACKGROUND}
+    @param layer: Layer on which the event occurred
     @type layer: integer
     '''
     #print "GaimPerk::executeTableRowInserted()", por, first_child_por, \
@@ -669,8 +677,7 @@ class BuddySignOnOff(Task.TableTask):
     @type first_child_por: L{POR}
     @param last_child_por: The L{POR} of last deleted row
     @type last_child_por: L{POR}
-    @param layer: Layer on which the event occurred, one of L{Task.LAYER_FOCUS},
-      L{Task.LAYER_TIER}, or L{Task.LAYER_BACKGROUND}
+    @param layer: Layer on which the event occurred
     @type layer: integer
     '''
     #print "GaimPerk::executeTableRowDeleted()" , por, first_child_por, \
