@@ -3,6 +3,7 @@ Defines L{Tools} for registering, unregistering, introspecting, and otherwise
 manging L{Tier}s, L{Perk}s, and L{Task}s.
 
 @author: Peter Parente
+@author: Scott Haeger
 @organization: IBM Corporation
 @copyright: Copyright (c) 2005, 2007 IBM Corporation
 @license: The BSD License
@@ -76,6 +77,15 @@ class System(Base.TaskTools):
     '''
     self.tier.popPerk(*indices)
     
+  def popThisPerk(self):
+    '''
+    Removes the current L{Perk} from the L{Tier}. No L{Task.Tools} methods 
+    should be invoked after this call.
+    '''
+    perks = self.tier.getPerks()
+    i = perks.index(self.perk)
+    self.tier.popPerk(i)
+    
   def reloadPerks(self):
     '''
     Reloads all L{Perk}s in the current L{Tier}. Any changes made to the 
@@ -101,6 +111,60 @@ class System(Base.TaskTools):
     self.tier = None
     self.acc_eng = None
     
+  def _getUIEInstances(self, kind, which):
+    '''
+    Internal method for getting instances of all installed or associated
+    L{UIElement}s for the given kind.
+    
+    @note: Installed UIEs that are not importable because of missing 
+      dependencies or other errors are not included in the list.
+    @note: Getting loaded chooser names is not currently supported.
+    @param kind: Kind of L{UIElement} to fetch metadata for, one of 
+      L{UIRegistrar.ALL_KINDS}
+    @type kind: string
+    @param which: Which names to get, one of the UIE_ constants in 
+      L{AEConstants}
+    @type which: integer
+    @return: All descriptions
+    @rtype: list of string
+    '''
+    reg = UIRegistrar
+    if which == AEConstants.UIE_LOADED:
+      # handle each type serparately, as loaded instances are stored in diverse
+      # locations in LSR
+      if kind == reg.PERK:
+        # get Perks in this Tier
+        uies = self.tier.getPerks()
+      elif kind == reg.DEVICE:
+        # get devices
+        uies = self.device_man.getDevices()
+      elif kind == reg.CHOOSER:
+        # not supported at present
+        uies = []
+      elif kind == reg.MONITOR:
+        imons, omons = self.device_man.getMonitors()
+        uies = list(set(imons) + set(omons) + set(self.tier_man.getMonitors()) 
+                    + set(self.event_man.getMonitors()))
+      # we already have live instances here, so just return them
+      return uies
+    
+    # use the UIRegistrar to look up class names
+    if which == AEConstants.UIE_INSTALLED:
+      names = reg.listInstalled(kind)
+    elif which == AEConstants.UIE_ALL_ASSOCIATED:
+      names = reg.listAssociated(kind, self.acc_eng.getProfile())
+    elif which == AEConstants.UIE_TIER_ASSOCIATED and kind == reg.PERK:
+      names = reg.listAssociated(kind, self.acc_eng.getProfile(),
+                             self.tier.getName())
+    # here we need to contruct live instances because all we have is a class 
+    # name from the UIRegistrar
+    l = []
+    for name in names:
+      uie = reg.loadOne(name)
+      if uie is not None:
+        l.append(uie)
+    return l
+
   def _getClassNames(self, kind, which):
     '''
     Internal method for getting class names of all installed or associated
@@ -193,17 +257,16 @@ class System(Base.TaskTools):
     
     The default value is used if which is invalid.
     
-    @todo: PP: optimize
-    
     @param which: Which names to get, one of the UIE_ constants in 
       L{AEConstants}
     @type which: integer
     @return: Class names, names, descriptions
     @rtype: list of 3-tuple of string
     '''
-    return zip(self.getPerkClassNames(which),
-               self.getPerkNames(which),
-               self.getPerkDescriptions(which))
+    uies = self._getUIEInstances(UIRegistrar.PERK, which)
+    return zip((uie.getClassName() for uie in uies),
+               (uie.getName() for uie in uies),
+               (uie.getDescription() for uie in uies))  
 
   def getDeviceMetadata(self, which=AEConstants.UIE_LOADED):
     '''    
@@ -213,17 +276,16 @@ class System(Base.TaskTools):
 
     The default value is used if which is invalid.
     
-    @todo: PP: optimize
-    
     @param which: Which names to get, one of the UIE_ constants in 
       L{AEConstants}
     @type which: integer
     @return: Class names, names, descriptions
     @rtype: list of 3-tuple of string
     '''
-    return zip(self.getDeviceClassNames(which), 
-               self.getDeviceNames(which),
-               self.getDeviceDescriptions(which))
+    uies = self._getUIEInstances(UIRegistrar.DEVICE, which)
+    return zip((uie.getClassName() for uie in uies),
+               (uie.getName() for uie in uies),
+               (uie.getDescription() for uie in uies))  
 
   def getMonitorMetadata(self, which=AEConstants.UIE_LOADED):
     '''    
@@ -232,26 +294,23 @@ class System(Base.TaskTools):
 
     The default value is used if which is invalid.
     
-    @todo: PP: optimize
-    
     @param which: Which names to get, one of the UIE_ constants in 
       L{AEConstants}
     @type which: integer
     @return: Class names, names, descriptions
     @rtype: list of 3-tuple of string
     '''
-    return zip(self.getMonitorClassNames(which), 
-               self.getMonitorNames(which),
-               self.getMonitorDescriptions(which))
-  
+    uies = self._getUIEInstances(UIRegistrar.MONITOR, which)
+    return zip((uie.getClassName() for uie in uies),
+               (uie.getName() for uie in uies),
+               (uie.getDescription() for uie in uies))  
+
   def getChooserMetadata(self, which=AEConstants.UIE_ALL_ASSOCIATED):
     '''        
     Gets metadata for all L{AEChooser}s installed in or associated with the
     active profile depending on the which flag.
     
     The default value is used if which is invalid.
-
-    @todo: PP: optimize
     
     @note: Getting loaded chooser names is not currently supported.   
     @param which: Which names to get, one of the UIE_ constants in 
@@ -260,9 +319,10 @@ class System(Base.TaskTools):
     @return: Class names, names, descriptions
     @rtype: list of 3-tuple of string
     '''
-    return zip(self.getChooserClassNames(which), 
-               self.getChooserNames(which),
-               self.getChooserDescriptions(which))
+    uies = self._getUIEInstances(UIRegistrar.CHOOSER, which)
+    return zip((uie.getClassName() for uie in uies),
+               (uie.getName() for uie in uies),
+               (uie.getDescription() for uie in uies))  
 
   def getDeviceClassNames(self, which=AEConstants.UIE_LOADED):
     '''
@@ -338,15 +398,9 @@ class System(Base.TaskTools):
     if which != AEConstants.UIE_LOADED:
       return self._getClassNames(UIRegistrar.MONITOR, which)
     else:
-      text = []
-      imons, omons = self.device_man.getMonitors()
-      mons = (imons, omons, self.tier_man.getMonitors(), 
-              self.event_man.getMonitors())
-      for col in mons:
-        for mon in col:
-          text.append(mon.getClassNames())
-      return text
-    
+      return [mon.getClassName() for mon in 
+              self._getUIEInstances(UIRegistrar.MONITOR, which)]
+
   def getMonitorNames(self, which=AEConstants.UIE_LOADED):
     '''    
     Gets the human readable names of all L{AEMonitor}s currently loaded in,
@@ -364,14 +418,8 @@ class System(Base.TaskTools):
     if which != AEConstants.UIE_LOADED:
       return self._getNames(UIRegistrar.MONITOR, which)
     else:
-      text = []
-      imons, omons = self.device_man.getMonitors()
-      mons = (imons, omons, self.tier_man.getMonitors(), 
-              self.event_man.getMonitors())
-      for col in mons:
-        for mon in col:
-          text.append(mon.getNames())
-      return text
+      return [mon.getName() for mon in 
+              self._getUIEInstances(UIRegistrar.MONITOR, which)]
     
   def getMonitorDescriptions(self, which=AEConstants.UIE_LOADED):
     '''    
@@ -390,14 +438,8 @@ class System(Base.TaskTools):
     if which != AEConstants.UIE_LOADED:
       return self._getDescriptions(UIRegistrar.MONITOR, which)
     else:
-      text = []
-      imons, omons = self.device_man.getMonitors()
-      mons = (imons, omons, self.tier_man.getMonitors(), 
-              self.event_man.getMonitors())
-      for col in mons:
-        for mon in col:
-          text.append(mon.getDescription())
-      return text
+      return [mon.getDescription() for mon in 
+              self._getUIEInstances(UIRegistrar.MONITOR, which)]
   
   def getChooserClassNames(self, which=AEConstants.UIE_ALL_ASSOCIATED):
     '''    
@@ -701,7 +743,7 @@ class System(Base.TaskTools):
       return None
     return dev.getDefaultStyle()
   
-  def getPerkState(self, perk_name):
+  def getPerkState(self, perk_name, reload=True):
     '''
     Gets the L{AEState} associated with a L{Perk}. If the state was previously
     persisted to disk, it is loaded from disk. If not, a new state object
@@ -713,6 +755,10 @@ class System(Base.TaskTools):
     @return: Loaded state for the L{Perk}
     @rtype: L{AEState}
     '''
+    if not reload:
+      perk = self.getPerkByName(perk_name)
+      return perk.getState()
+    
     # load an instance of the perk
     perk = UIRegistrar.loadOne(perk_name)
     if perk is None:
@@ -838,8 +884,7 @@ class System(Base.TaskTools):
     # unregister the task observing this chooser
     self.perk.unregisterChooserTask(chooser)
 
-  def doTask(self, task_name, chain=True, propagate=True, layer=None,
-             **kwargs):
+  def doTask(self, task_name, chain=True, propagate=True, layer=None,**kwargs):
     '''
     Immediately executes a L{Task} registered under the given name in this
     L{Tier}. Keyword arguments for L{Task} execution may be provided. The 
@@ -870,6 +915,7 @@ class System(Base.TaskTools):
       may be completely ignored
     @type kwargs: dictionary
     '''
+    # use the current layer if one is not specified
     if layer is None: layer = self.layer
     self.tier.manageNamedTask(self.task_por, task_name, layer, kwargs,
                               chain=chain)
@@ -902,7 +948,7 @@ class System(Base.TaskTools):
           self.unregisterTask(self)
         return False
     # register an instance as a blocker
-    self.registerTask(consumer())
+    self.registerTask(consumer(None))
     
   def associatePerk(self, name, tier=None, all_tiers=False, index=None):
     '''
@@ -1043,4 +1089,234 @@ class System(Base.TaskTools):
     @raise KeyError: When one of the named constants does not exist in the 
       scope
     '''
-    AEConstants.register(dict(((name, scope[name]) for name in names)))
\ No newline at end of file
+    AEConstants.register(dict(((name, scope[name]) for name in names)))
+     
+  def getPerkByName(self, name):
+    '''
+    Gets the L{Perk} with the given name.
+    
+    If no perk is registered under the given name, KeyError is raised.
+
+    @param name: Name of the perk
+    @type name: string
+    @return: Perk with the given name
+    @rtype: object
+    @raise KeyError:  When the perk with the given name is not found
+    '''
+    for perk in self.tier.getPerks():
+      if perk.getClassName() == name:
+        return perk
+    raise KeyError
+    
+  def getPerkVar(self, perk_name, var_name):
+    '''
+    Gets the L{Perk} instance variable with the given name.
+    
+    If no perk is registered under the given name a KeyError is raised
+    if the perk does not contain var_name an AttributeError is raised.
+
+    @param perk_name: Name of the perk
+    @type perk_name: immutable
+    @param var_name: Name of the instance variable
+    @type var_name: immutable
+    @return: perk variable with the given name
+    @rtype: object
+    @raise KeyError:  When the perk or variable with it's given name 
+      is not found
+    @raise AttributeError:  When attribute is not found
+    is not found
+    '''
+    perk = self.getPerkByName(perk_name)
+    return getattr(perk, var_name)
+  
+  def setPerkVar(self, perk_name, var_name, value):
+    '''
+    Sets the L{Perk} instance variable with the given name to value.
+    
+    If no perk is registered under the given name a KeyError is raised
+    if the perk does not contain var_name an AttributeError is raised.
+
+    @param perk_name: Name of the perk
+    @type perk_name: immutable
+    @param var_name: Name of the instance variable
+    @type var_name: immutable
+    @param value: value to set instance variable
+    @type value: immutable
+    @return: perk variable with the given name
+    @rtype: object
+    @raise KeyError:  When the perk or variable with it's given name 
+    @raise AttributeError:  When attribute is not found
+    '''
+    perk = self.getPerkByName(perk_name)
+    setattr(perk, var_name, value)
+  
+  def getPerkSetting(self, setting_name, perk_name=None):
+    '''
+    Gets the L{Perk} L{Setting} state variable with the given name.
+    
+    If no perk is registered under the given name or L{setting_name} is not
+    found, KeyError is raised.
+
+    @param perk_name: Name of the perk
+    @type perk_name: immutable
+    @param setting_name: Name of the perk
+    @type setting_name: immutable
+    @return: L{Setting} object for given perk.
+    @rtype: object
+    @raise KeyError:  When the perk is not found
+    '''
+    if perk_name is None:
+      perk_name = self.perk.getClassName()
+    state = self.getPerkState(perk_name, reload=False)
+    return state.getSettingObj(setting_name)
+  
+  def getPerkSettingVal(self, setting_name, perk_name=None):
+    '''
+    Gets the value of the L{Perk} L{Setting} state variable with the given 
+    name.
+    
+    If no perk is registered under the given name or if L{setting_name} is not 
+    found, KeyError is raised.
+
+    @param perk_name: Name of the perk
+    @type perk_name: immutable
+    @param setting_name: Name of the perk
+    @type setting_name: immutable
+    @return: value contained in L{AEState} attribute with given name
+    @rtype: object
+    @raise KeyError:  When the perk is not found
+    '''
+    if perk_name is None:
+      perk_name = self.perk.getClassName()
+    state = self.getPerkState(perk_name, reload=False)
+    return state.getSettingVal(setting_name)
+  
+  def setPerkSettingVal(self, setting_name, value, perk_name=None):
+    '''
+    Sets the value of the L{Perk} L{Setting} state variable with the given 
+    name.
+    
+    If no perk is registered under the given name or if L{setting_name} is not 
+    found, KeyError is raised.
+
+    @param perk_name: Name of the perk
+    @type perk_name: immutable
+    @param setting_name: Name of the perk
+    @type setting_name: immutable
+    @value: value contained in L{AEState} attribute with given name
+    @value: object
+    @raise KeyError:  When the perk is not found
+    '''
+    if perk_name is None:
+      perk_name = self.perk.getClassName()
+    state = self.getPerkState(perk_name, reload=False)
+    state.setSettingVal(setting_name, value)
+    
+  def getHistory(self):
+    '''
+    Gets the zipped contents of L{getTaskHistory} and L{getPerkHistory}. This
+    method does not involve extra processing because the history is already
+    zipped in the L{Tier}. In fact, the other methods require the extra work
+    of unzipping.
+    
+    The history is ordered from the first L{Task} to respond to an event to
+    the most recent L{Task} to respond, including the current L{Task}.
+    
+    @return: List of L{Task} identities and L{Perk} class names
+    @rtype: list of 2-tuple of string
+    '''
+    return self.tier.getHistory()
+    
+  def getTaskHistory(self):
+    '''
+    Gets the identities of all L{Task}s that have executed in response to the
+    event currently being processed. This includes all chained L{Task}s and all
+    L{Task}s executed using L{doTask} flattened into a linear history.
+    
+    If any L{Task} that executed did not have an identity, None is substituted
+    in its place.
+    
+    The history is ordered from the first L{Task} to respond to an event to
+    the most recent L{Task} to respond, including the current L{Task}.
+    
+    @return: Identities of the L{Task}s
+    @rtype: list of string
+    '''
+    h = self.tier.getHistory()
+    return [names[0] for names in h]
+    
+  def getPerkHistory(self):
+    '''
+    Gets the class names of all L{Perk}s that have executed in response to the
+    event currently being processed. The contents of the list returned by this
+    method have a one-to-one pairing with the list contents from 
+    L{getTaskHistory}.
+    
+    The history is ordered from the first L{Perk} to respond to an event to
+    the most recent L{Perk} to respond, including the current L{Perk}.
+      
+    @return: Class names of the L{Perk}s
+    @rtype: list of string
+    '''
+    h = self.tier.getHistory()
+    return [names[1] for names in h]
+  
+  def getAnchorTaskId(self):
+    '''
+    Gets the identity of the L{Task} which this one is executing before, after, 
+    or around according to a previous chaining. If this L{Task} is not 
+    executing as part of a chain, returns None.
+    
+    @return: Identity of the L{Task} or None
+    @rtype: string
+    '''
+    try:
+      return self.tier.getAnchor()[0]
+    except TypeError:
+      return None
+  
+  def getAnchorPerkClassName(self):
+    '''
+    Gets the class name of the L{Perk} owning the L{Task} that would be 
+    returned by L{getAnchorTaskId}.
+    
+    @see: L{getAnchorTaskId}
+    @return: Class name of the L{Perk} or None
+    @rtype: string
+    '''
+    try:
+      return self.tier.getAnchor()[1]
+    except TypeError:
+      return None
+
+  def getPerkTaskNames(self, perk_name=None):
+    '''
+    Gets the names of all L{Task}s currently registered in the named L{Perk}.
+    If no name is given, this L{Perk} is used. If the named L{Perk} is not 
+    found, an exception is raised.
+    
+    @param perk_name: Name of the L{Perk} to query or None to mean this one
+    @type perk_name: string
+    @return: List of all registered L{Task} names
+    @rtype: list of string
+    @raise KeyError: When the named L{Perk} is not loaded
+    '''
+    if perk_name is None:
+      return self.perk.getTaskNames()
+    else:
+      for perk in self.tier.getPerks():
+        if perk.getClassName() == perk_name:
+          return perk.getTaskNames()
+      raise KeyError(perk_name)
+  
+  def getTierTaskNames(self):
+    '''
+    Gets the names of all L{Task}s currently registered in this L{Tier}.
+    
+    @return: List of all registered L{Task} names
+    @rtype: list of string
+    '''
+    names = []
+    for perk in self.tier.getPerks():
+      names.extend(perk.getTaskNames())
+    return names
